/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: SF_PREVISAO_DE_CONSUMO_PLA

Descrio:
Clculo da Previso de Consumo atravs da stored procedure CONSUMO_PLANEJADO e
gravao na tabela PREVCONS.

P_CDFILIAL     - Codigo da filial
P_SERVICOS     - String contendo sequncia de servios a serem calculados
P_DTINICIAL    - Data inicial de clculo
P_DTFINAL      - Data final de clculo
P_PEACRESCIMO  - Percentual de acrscimo
P_CDPRODUTOINI - Cdigo do produto inicial (CDARVPROD)
P_CDPRODUTOFIN - Cdigo do produto final   (CDARVPROD)
P_CDOPERADOR   - Cdigo do operador que solicitou o clculo
P_DTHRCALCPC   - Data/hora do clculo
P_NRCONSPLAN   - Nmero sequencial a ser utilizado para gravao dos dados na
                 tabela CONSPLAN (tabela de trabalho)
P_STDTINICIAL  - Lista de data inicial (em string) por servio
P_STDTFINAL    - Lista de data final (em string) por servio
P_NRORG        - Nmero da organizao

Observao: o formato das strings de data inicial e de data final deve ser
"_YYYYMMDD_YYYYMMDD_YYYYMMDD_"
"_YYYYMMDD_YYYYMMDD_YYYYMMDD_"


Para aplicar e debugar
-- @C:\WIN_PRJ\SQL\ORACLE\SF_7130_PLASTP_PREVISAO_DE_CONSUMO_PLA.SQL;
-- START H:\WORKDIR\Ederson\BACKUP\MANUTENO\SF_7130_PLASTP_PREVISAO_DE_CONSUMO_PLA.SQL;
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');

Criao:
14/03/2012 - Ederson Melquades da Cunha

*/

CREATE OR REPLACE 
PROCEDURE SF_PREVISAO_DE_CONSUMO_PLA
  (P_CDFILIAL     IN VARCHAR2,
   P_SERVICOS     IN VARCHAR2,
   P_DTINICIAL    IN DATE,
   P_DTFINAL      IN DATE,
   P_PEACRESCIMO  IN NUMBER,
   P_CDPRODUTOINI IN VARCHAR2,
   P_CDPRODUTOFIN IN VARCHAR2,
   P_CDOPERADOR   IN VARCHAR2,
   P_DTHRCALCPC   IN DATE,
   P_NRCONSPLAN   IN VARCHAR2,
   P_IDARREDOND   IN VARCHAR2,
   P_STDTINICIAL  IN VARCHAR2 DEFAULT NULL,
   P_STDTFINAL    IN VARCHAR2 DEFAULT NULL,
   P_NRORG        IN PREVCONS.NRORG%TYPE DEFAULT 1) IS

   V_DTINICIAL    DATE;
   V_DTFINAL      DATE;
   V_COUNTSERVICO NUMBER;
   V_CDARVINI     VARCHAR2(20);
   V_CDARVFIN     VARCHAR2(20);

   V_IDCOMCUSPERC VARCHAR2(01);
   V_IDCALPADCMP  VARCHAR2(01);
   V_CDTPCUSTCMP  VARCHAR2(02);
   V_INTPCUSTCMP  NUMBER;
   V_VRINDCALCMP  NUMBER;
   V_QTCOMES      NUMBER;
   V_VRCUSTO      NUMBER;
   V_VRCUSTOPERC  NUMBER;
   V_VRCUSTOPADR  NUMBER;
   V_CDFILIAL     VARCHAR2(04);
   V_CDSERVFILI   VARCHAR2(05);
   V_IDUTCARDPADR VARCHAR2(01);
   V_CDPRODUTOINI VARCHAR2(10);
   V_CDPRODUTOFIN VARCHAR2(10);
BEGIN
   V_CDARVINI := P_CDPRODUTOINI;
   V_CDARVFIN := RPAD(P_CDPRODUTOFIN, 20, 'z');

   SELECT MAX(IDUTCARDPADR)
      INTO V_IDUTCARDPADR
   FROM PARAMGERAL;
   IF (V_IDUTCARDPADR = 'S') THEN
      SELECT MAX(CDFILIAL)
      INTO   V_CDFILIAL
      FROM   FILIAL
      WHERE  IDFILCENTPME = 'S' AND
             CDFILIAL = P_CDFILIAL AND
             CDFILCENTPME IS NULL;
      IF V_CDFILIAL IS NOT NULL THEN
         RETURN;
      END IF;

      SELECT MIN(CDSERVFILI)
      INTO   V_CDSERVFILI
      FROM
         (SELECT
             S.CDSERVFILI,
             NVL(C.IDCARDLIBAGRE, 'N') AS IDCARDLIBAGRE
          FROM
             SERVICO S,
             CARDAMES C
          WHERE
             S.CDFILIAL = P_CDFILIAL AND
             ((INSTR(P_SERVICOS, S.CDSERVFILI) <> 0) OR (P_SERVICOS = 'T')) AND
             S.CDFILIAL     = C.CDFILIAL(+)   AND
             S.CDSERVFILI   = C.CDSERVFILI(+) AND
             (C.DTMESCARDAPIO(+) BETWEEN P_DTINICIAL AND P_DTFINAL)
             AND IDTIPOSERV = 'N')
      WHERE  IDCARDLIBAGRE = 'N';
      IF (V_CDSERVFILI IS NOT NULL) THEN
         Raise_Application_Error(-20502, 'Cardpio no liberado para produo. Servio: '||V_CDSERVFILI, False);
      END IF;
   END IF;

   SELECT MAX(CDPRODUTO) INTO V_CDPRODUTOINI FROM PRODUTO WHERE CDARVPROD = P_CDPRODUTOINI;
   SELECT MAX(CDPRODUTO) INTO V_CDPRODUTOFIN FROM PRODUTO WHERE CDARVPROD = P_CDPRODUTOFIN;

   /* - REGISTRO DO LOG E EXCLUSO DOS REGISTROS ANTERIORES DA PREVCONS ----- */
   V_COUNTSERVICO := 2;
   FOR CUR_SERVICO IN
      (SELECT
          CDSERVFILI
       FROM
          SERVICO
       WHERE
          (CDFILIAL   = P_CDFILIAL) AND
          (IDSERVATIV = 'S')        AND
          ((INSTR(P_SERVICOS, CDSERVFILI) > 0) OR P_SERVICOS = 'T')
       ORDER BY
           CDSERVFILI)
   LOOP

      IF P_STDTINICIAL IS NULL THEN
         V_DTINICIAL    := P_DTINICIAL;
         V_DTFINAL      := P_DTFINAL;
      ELSE
         V_DTINICIAL    := TO_DATE(SUBSTR(P_STDTINICIAL, V_COUNTSERVICO, 8), 'YYYYMMDD');
         V_DTFINAL      := TO_DATE(SUBSTR(P_STDTFINAL, V_COUNTSERVICO, 8), 'YYYYMMDD');
         V_COUNTSERVICO := V_COUNTSERVICO + 9;
      END IF;

      INSERT INTO LOGPREVCONS
         (CDFILIAL,
          CDOPERADOR,
          DTHRCALCPC,
          NRCONSPLANPC,
          CDSERVFILI,
          DTINICIALPC,
          DTFINALPC,
          CDPRODINICPC,
          CDPRODFINAPC,
          PEACRESCIMPC)
      VALUES
         (P_CDFILIAL,
          P_CDOPERADOR,
          P_DTHRCALCPC,
          P_NRCONSPLAN,
          CUR_SERVICO.CDSERVFILI,
          V_DTINICIAL,
          V_DTFINAL,
          V_CDPRODUTOINI,
          V_CDPRODUTOFIN,
          P_PEACRESCIMO);

      /* - APAGA OS REGISTROS ANTERIORES DA TABELA PREVCONS ----------------- */
      DELETE FROM PREVCONS
      WHERE
         (CDFILIAL   = P_CDFILIAL)                      AND
         (CDSERVFILI = CUR_SERVICO.CDSERVFILI)          AND
         (DTPREVCONS BETWEEN V_DTINICIAL AND V_DTFINAL) AND
         (EXISTS (SELECT CDPRODUTO
                    FROM PRODUTO
                   WHERE CDARVPROD BETWEEN V_CDARVINI AND V_CDARVFIN
                           AND PRODUTO.CDPRODUTO = PREVCONS.CDPRODUTO));
   END LOOP; /* LOOP DOS SERVIOS */

   COMMIT;

   /* CALCULA OS DADOS ATRAVS DA STORED PROCEDURE CONSUMO PLANEJADO */

   CONSUMO_PLANEJADO_PLA(P_CDFILIAL,
                         P_SERVICOS,
                         P_DTINICIAL,
                         P_DTFINAL,
                         'C',   /* GRAVAO NA TABELA CONSPLAN */
                         'CFP', /* CARDPIO, CONSUMO FIXO E PERCAPITA */
                         'N',   /* NO SEPARAR POR SETOR */
                         P_PEACRESCIMO,
                         P_IDARREDOND,
                         P_NRCONSPLAN,
                         V_CDARVINI,
                         V_CDARVFIN,
                         P_CDOPERADOR,
                         P_STDTINICIAL,
                         P_STDTFINAL);

   COMMIT;

   SELECT MAX(IDCALPADCMP), MAX(CDTPCUSTCMP), MAX(VRINDCALCMP) INTO
          V_IDCALPADCMP, V_CDTPCUSTCMP, V_VRINDCALCMP
   FROM PARAMPLANEJA
   WHERE (CDFILIAL = P_CDFILIAL);

   V_INTPCUSTCMP := TO_NUMBER(V_CDTPCUSTCMP);

   SELECT MAX(IDCOMCUSPERC) INTO V_IDCOMCUSPERC
   FROM PARAMFILIAL
   WHERE (CDFILIAL = P_CDFILIAL);

   IF (V_IDCALPADCMP = 'S') THEN
      FOR CUR_SERVICO IN
         (SELECT CDFILICPLA AS CDFILIAL,
                 CDSERVCPLA AS CDSERVFILI
          FROM CONSPLAN
          WHERE
             (CDFILICPLA = P_CDFILIAL) AND
             (NRCONSPLAN = P_NRCONSPLAN)
          GROUP BY CDFILICPLA,
                   CDSERVCPLA
      ) LOOP
         V_VRCUSTO := 0.0;
         SELECT
            SUM(CUSTO_PRODUTO(CONSPLAN.CDFILICPLA, CONSPLAN.CDPRODCPLA, V_INTPCUSTCMP, CONSPLAN.DTCONSPLAN, SYSDATE) * CONSPLAN.QTCONSPLAN) INTO V_VRCUSTO
         FROM CONSPLAN
         WHERE (CONSPLAN.CDFILICPLA = P_CDFILIAL)              AND
               (CONSPLAN.NRCONSPLAN = P_NRCONSPLAN)            AND
               (CONSPLAN.CDFILICPLA = CUR_SERVICO.CDFILIAL)    AND
               (CONSPLAN.CDSERVCPLA = CUR_SERVICO.CDSERVFILI);

         IF (V_VRCUSTO IS NULL) THEN V_VRCUSTO := 0.0; END IF;

         SELECT SUM(NVL(VRCUSTPADR * V_VRINDCALCMP, 0.0)) INTO V_VRCUSTOPADR
         FROM CUSTPADR
         WHERE
            (CDFILIAL   = CUR_SERVICO.CDFILIAL)   AND
            (CDSERVFILI = CUR_SERVICO.CDSERVFILI) AND
            (TO_CHAR(DTREFCUSPADR, 'MM/YYYY') = TO_CHAR(P_DTFINAL, 'MM/YYYY'));

         SELECT
            SUM(DECODE(V_IDCOMCUSPERC,
                       'C', QTCLIESTICAR,
                            QTCOMESTICAR)) INTO V_QTCOMES
         FROM
            CARDADIA
         WHERE
            (CDFILIAL   = CUR_SERVICO.CDFILIAL)   AND
            (CDSERVFILI = CUR_SERVICO.CDSERVFILI) AND
            (DTCARDAPIO BETWEEN P_DTINICIAL AND P_DTFINAL);

         V_VRCUSTOPERC := 0;
         IF (V_QTCOMES <> 0.0) AND (V_VRCUSTO <> 0.0) THEN
            V_VRCUSTOPERC := (V_VRCUSTO/V_QTCOMES);
         END IF;

         IF (V_VRCUSTOPADR > V_VRCUSTOPERC) THEN
            /* INSERE NA TABELA PREVCONS AS INFORMAES CALCULADAS NA CONSPLAN */
            INSERT INTO PREVCONS
               (CDFILIAL,
                CDSERVFILI,
                CDPRODUTO,
                DTPREVCONS,
                QTPREVCONS)
            SELECT
               CDFILICPLA,
               CDSERVCPLA,
               CDPRODCPLA,
               DTCONSPLAN,
               SUM(QTCONSPLAN)
            FROM CONSPLAN
            WHERE(CDFILICPLA = P_CDFILIAL)             AND
                 (NRCONSPLAN = P_NRCONSPLAN)           AND
                 (CDFILICPLA = CUR_SERVICO.CDFILIAL)   AND
                 (CDSERVCPLA = CUR_SERVICO.CDSERVFILI) AND
                 EXISTS (SELECT CDSERVCPLA
                           FROM SERVICO
                          WHERE CONSPLAN.CDFILICPLA = SERVICO.CDFILIAL
                            AND CONSPLAN.CDSERVCPLA = SERVICO.CDSERVFILI
                            AND SERVICO.IDSERVATIV = 'S')

            GROUP BY
               CDFILICPLA,
               CDSERVCPLA,
               CDPRODCPLA,
               DTCONSPLAN;
          END IF;
      END LOOP;
   ELSE
      /* INSERE NA TABELA PREVCONS AS INFORMAES CALCULADAS NA CONSPLAN */
      INSERT INTO PREVCONS
         (CDFILIAL,
          CDSERVFILI,
          CDPRODUTO,
          DTPREVCONS,
          QTPREVCONS)
         SELECT
            CDFILICPLA,
            CDSERVCPLA,
            CDPRODCPLA,
            DTCONSPLAN,
            SUM(QTCONSPLAN)
         FROM
            CONSPLAN
         WHERE
            CDFILICPLA = P_CDFILIAL   AND
            NRCONSPLAN = P_NRCONSPLAN AND
            EXISTS (SELECT CDSERVCPLA
                      FROM SERVICO
                     WHERE CONSPLAN.CDFILICPLA = SERVICO.CDFILIAL
                       AND CONSPLAN.CDSERVCPLA = SERVICO.CDSERVFILI
                       AND SERVICO.IDSERVATIV = 'S')
         GROUP BY
            CDFILICPLA,
            CDSERVCPLA,
            CDPRODCPLA,
            DTCONSPLAN;
   END IF;

   COMMIT;

   /* APAGA OS DADOS DA TABELA DE TRABALHO UTILIZADA NO CLCULO*/
   DELETE FROM CONSPLAN
   WHERE
      CDFILICPLA = P_CDFILIAL AND
      NRCONSPLAN = P_NRCONSPLAN;

   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('SF_PREVISAO_DE_CONSUMO_PLA','0.00.000'); END;
/